// SystoolSamurize.cpp : Defines the entry point for the DLL application.
//

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#include "stdafx.h"
#include <comutil.h>
#include <windows.h>
#include "SystoolMem.h"
#include "resource.h"
#include <string>
#include <sstream>

using namespace std;

#define SAMURIZE_PLUGIN_CHAR  extern "C" __declspec(dllexport) char* __stdcall
#define SAMURIZE_PLUGIN_INT   extern "C" __declspec(dllexport) int __stdcall

HWND hSamurize;

struct meter_data
{
	string value;
};
void meter_reset(meter_data* tMeter)
{
	tMeter->value = "Unknown";
}

HANDLE hInstance;

// you can use this function to do one-time initialization and de-initialization
// remember, dllstartup and dllshutdown are called PER METER !!!
BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call) {
		case DLL_PROCESS_ATTACH: 
			// this one is called as soon as samurize loads the dll
			// do global initialization stuff here
			hInstance = hModule;
            break;

		case DLL_PROCESS_DETACH:
			// this is called as samurize unloads the dll
			// free/delete all your data
			break;
		// these two are never called from samurize
		//case DLL_THREAD_ATTACH:  
		//case DLL_THREAD_DETACH:  
	}
        return TRUE;
}

// additionally to the bunch of modifiers for the functions, you
// have to make sure that they are exported "undecorated" as delphi
// doesn't like decorated functions (and samurize is programmed in
// delphi).
// for Microsoft Visual Studio Compilers, you have to add /EXPORT
// statements to the linker commandline. go to the project options
// and add /EXPORT:init /EXPORT:getparam and so on manually.
// For other compilers you need to use a DEF file one is included
// with this example to see what they look like.  Use pexports.exe
// to find out what the decorated names are and them add them to the
// defs file.
SAMURIZE_PLUGIN_CHAR init()
{	
	// ok here you will return the list of the functions, separated by 
	// a | if there are more than one like
	// that somefunc|someotherfunc|alastfunc
	return "ValueBySensor|ValueByName";
}

//Samurize GetParam, returns a list of the parameters that each function needs
SAMURIZE_PLUGIN_CHAR getparam(char* func)
{
	//here you should have an if statement or a case to determnie what was function asking for params
	//the func param is the config editor asking for the parameter list of a funcion
	//so return the tex separated by | and this time it must finish with a |
	//ie : param1|param2|
	//or must be empty like in this template when there are no parameters
	if(!stricmp(func, "ValueBySensor"))
	{
		return "sensor number|";
	}
	else if (!stricmp(func, "ValueByname"))
	{
		return "sensor name|";
	}
    else
    {
		return "|";
	}
}

// dllstartup() is called every time Samurize creates a meter that uses this plugin.
// The 'hwnd' parameter is the window handle of the instance of Samurize that called this function.
// 'hwnd' will be 0 if called from the config editor.
// The 'dlltype' parameter indicates the meter 'type' (1 for source plugins, 2 for visual plugins)
// that is currently initialized.
// This function should return a unique integer ID that Samurize remembers and uses for other functions
// (this is so multiple instances of Samurize that use this do not conflict).
//
// Any memory allocation needed on a meter-by-meter basis should be done in this function.
//
// If your plugin is very simple and does not need to keep any global variables that may cause conflicts,
// just return 0 (or omit this function).
SAMURIZE_PLUGIN_INT dllstartup(HWND hwnd, int dlltype)
{
	//Save the Samurize Handle
    hSamurize				 = hwnd;

	//Create the Meter and Setup its values
	meter_data* tMeter		   = new meter_data;
	//Call the Meter reset function to initialise all the variables to there default values
	meter_reset(tMeter);
    
	//Cast the meter pointer to an integer and return to samurize
	return (int) tMeter;
}

// dllshutdown() is called every time a meter is destroyed by Samurize.
// The 'id' parameter is the uniqueID your plugin assigned to the meter (returned by dllstartup)
// This function should always return 0.
//
// Any memory releasing needed on a meter-by-meter basis should be done in this function.
//
// If your plugin does not need to free any memory, this function may be omitted.
SAMURIZE_PLUGIN_INT dllshutdown(int id)
{
    //Need to free the meter of the meter
	meter_data* tMeter = (meter_data*)id;

	//Delete the Meter to free the memory
	delete tMeter;

	//Always return a zero back to Samurize
	return 0;
}

// getinfo() should return information about your plugin depending on the passed integer value.
// 0: Plugin name
// 1: Author name
// 2: Plugin version
// 3: Creation date
// 4: Last modified date
// 5: Website
// 6: Contact email address
//
// This information is displayed when the 'About' button is pressed in the config editor when your plugin
// is selected.
//
// This function is optional.
SAMURIZE_PLUGIN_CHAR getinfo(int infonum)
{
	// called by samurize to display something useful in the about dialog.
	switch (infonum) {
        case 0: return "Systool Plugin - For Samurize Version 2 Interface % Systool Build 512";
		case 1: return "_xhp_";
        case 2: return "1.0a";
        case 3: return "22.07.05";
        case 4: return "22.07.05";
        case 5: return "";
        case 6: return "";
	}
	return "";
}

// DLLType: Tells Samurize what type of plugin this is.
// because this dlltype() function is declared, Samurize will assume you are using the SDK v2.0;
// 	* All functions will be called with an additional first parameter, the meter's unique_id
//
// Omit the dlltype() function to use the old SDK (the uniqueID will be omitted)
//
// The value you should return is a summation of any or all of the following values:
// 1: Source plugin
// 2: Visual plugin
// ... (more to follow!)
SAMURIZE_PLUGIN_INT dlltype()
{
    return 1;
	// you can also have a source plugin and a visual plugin in the same dll,
	// if so, return the sum of the plugin type numbers (that would be 3)
}

// declaration...
 _variant_t GetSensorValue(UINT16 sensor);

SAMURIZE_PLUGIN_CHAR ValueBySensor(int id, char* sSensorNumber)
{
	//Create the meter by casting the returned id value
	meter_data* tMeter = (meter_data*) id;
	try
	{
		UINT16 sensor = atoi(sSensorNumber);
		if (sensor == 0)
		{
			tMeter->value = "Invalid sensor number";
		}
		else
		{
			_variant_t v = GetSensorValue (sensor);
			tMeter->value = (char*)(_bstr_t)(v);
		}
	}
	catch( ... )
	{
		tMeter->value = "An unexpected error has happened";
	}
	return (char*)tMeter->value.c_str ();
}

UINT16 FindSensorByName(char* sSensorName)
{
	SHMEM* pShmem;
    HANDLE hSData;

    hSData = OpenFileMapping(FILE_MAP_READ, FALSE, SH_MEM_NAME);
    if (hSData == 0)
		return 0;
	pShmem = (SHMEM *)MapViewOfFile(hSData, FILE_MAP_READ, 0, 0, 0);
    if (pShmem == 0){
		CloseHandle(hSData);
		return 0;
    }

	UINT16 s = 0;
	if(pShmem->m_version != SH_MEM_VERSION)
	{
		s = 0;
	}
	else
	{
		for (int i = 0; i < pShmem->m_nSensors; i++)
		{
			char _name[255];
			wcstombs(&_name[0], pShmem->m_sensors[i].m_name, sizeof(_name));
			if (strcmp(_name, sSensorName) == 0)
			{
				s = i + 1;
				break;
			}
		}
	}

	UnmapViewOfFile(pShmem);
	CloseHandle(hSData);

	return s;
}

SAMURIZE_PLUGIN_CHAR ValueByName(int id, char* sSensorName)
{
	//Create the meter by casting the returned id value
	meter_data* tMeter = (meter_data*) id;
	try
	{
		UINT16 sensor = FindSensorByName(sSensorName);
		if (sensor == 0)
		{
			tMeter->value = "Invalid sensor number";
		}
		else
		{
			_variant_t v = GetSensorValue (sensor);
			tMeter->value = (char*)(_bstr_t)(v);
		}
	}
	catch( ... )
	{
		tMeter->value = "An unexpected error was returned";
	}
	return (char*)tMeter->value.c_str ();
}

_variant_t GetSensorValue(UINT16 sensor)
{
	sensor--; // from now on the real value

	SHMEM* pShmem;
    HANDLE hSData;

    hSData = OpenFileMapping(FILE_MAP_READ, FALSE, SH_MEM_NAME);
    if (hSData == 0)
		return "Systool is not running or unsupported version";
	pShmem = (SHMEM *)MapViewOfFile(hSData, FILE_MAP_READ, 0, 0, 0);
    if (pShmem == 0){
		CloseHandle(hSData);
		return "Can not read shared data";
    }

	_variant_t v;
	if(pShmem->m_version != SH_MEM_VERSION)
	{
		v = "Invalid Systool version detected";
	}
	else
	{
		if(sensor >= pShmem->m_nSensors || sensor >= SH_MEM_MAX_SENSORS-1)
		{
			v = "Invalid sensor number";
		}
		else
		{	
			SHMEM_SENSOR* pSensor = &pShmem->m_sensors[sensor];
			if (pSensor->m_updateInProgress == 1)
			{
				while (pSensor->m_updateInProgress == 1)
				{ /* ?? */ }
			}
			v = pSensor->m_value;
		}
	}

	UnmapViewOfFile(pShmem);
	CloseHandle(hSData);

	return v;
}

template<typename T>
wstring toString(T value)
{
    wostringstream s;
	s << value;
	return s.str ();
}

INT_PTR CALLBACK SettingsDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_INITDIALOG:
		{
			SHMEM* pShmem;
			HANDLE hSData;

			hSData = OpenFileMapping(FILE_MAP_READ, FALSE, SH_MEM_NAME);
			if (hSData == 0)
			{
				MessageBox(hSamurize, "Systool is not running or unsupported version", "Error", MB_OK);
				return TRUE;
			}

			pShmem = (SHMEM *)MapViewOfFile(hSData, FILE_MAP_READ, 0, 0, 0);
			if (pShmem == 0){
				CloseHandle(hSData);
				MessageBox(hSamurize, "Can not read shared data", "Error", MB_OK);
				return TRUE;
			}

			HWND listbox = GetDlgItem(hwndDlg, IDC_LIST);
			if(listbox != NULL)
			{
				for (int s = 0; s < pShmem->m_nSensors; s++)
				{
					SHMEM_SENSOR* sensor = &pShmem->m_sensors[s];
                    wstring item;
					item = toString(s+1) + L": " + sensor->m_name;
					SendMessageW(listbox, LB_ADDSTRING, 0, (LPARAM)item.c_str ());
				}
			}
			else
				MessageBox(hSamurize, "Error filling list", "Error", MB_OK);

			UnmapViewOfFile(pShmem);
			CloseHandle(hSData);

			return TRUE;
		}
		case WM_COMMAND:
			if(LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
				EndDialog(hwndDlg, IDOK);
	}
	return FALSE;
}

SAMURIZE_PLUGIN_INT configure()
{
	DialogBox((HINSTANCE)hInstance, MAKEINTRESOURCE(IDD_SETTINGS), hSamurize, SettingsDlgProc);
	return 0;
}
